//
//  Util.swift
//  zip_app
//
//  Created by Michiyuki Abe on 2019/02/15.
//  Copyright © 2019 zip_pro. All rights reserved.
//

import Foundation
import Cocoa

public class Util {
    
    let TRIAL_PERIOD = 60*60*24 // prod=60*60*24 (24 hours) dev=60*1 (1min) 60*60*24
    
    public init() {
    }
    
    internal func loadSettings() -> SettingStatus {
        var result = SettingStatus()
        let dict = UserDefaults.standard.dictionary(forKey: "settings")!
        if let val = dict["isEncrypt"] {
            result.isEncrypt = val as! Bool
        }
        if let val = dict["password"] {
            result.password = val as! String
        }
        if let val = dict["isSelectDir"] {
            result.isSelectDir = val as! Bool
        }
        return result
    }
    
    internal func loadToken() -> String? {
        if let dict = UserDefaults.standard.dictionary(forKey: "token"), let val = dict["token"] {
            return val as! String
        }
        return nil
    }
    
    internal func userIsTrialPeriod() -> Bool {
        // 最初にアプリを起動してから1日以内か否かを返す
        return UserDefaults.standard.integer(forKey: "firstStartedAt") + TRIAL_PERIOD > Int(NSDate().timeIntervalSince1970)
    }
    
    // 認証コードを入力するアラートを表示する関数
    // zipする前に呼び出す
    // viewDidLoad内で呼ぶとshowRecursiveAuthInputの中のbeginSheetModalでnilになりおエラー
    // viewDidLoadを通らないと、self.view.windowがエラーになると想定される
    // 参考）https://qiita.com/keroxp/items/31662245b92de3eea23f
    public func showAuthAlert(window: NSWindow) {
        let alert = NSAlert()
        let authInput = NSTextField(frame: NSRect(x: 0, y: 0, width: 300, height: 20))
        alert.alertStyle = NSAlert.Style.informational
        alert.messageText = "未認証"
        alert.informativeText = "購入した認証コードを入力して、アクティベーションしてください。詳しくは（https://zipforwin.com/）でご確認ください"
        alert.accessoryView = authInput
        alert.addButton(withTitle: "認証")
        alert.addButton(withTitle: "キャンセル")
        showRecursiveAuthInput(alert: alert, input: authInput, window: window)
    }
    // 認証コードが正しく入力されていない場合、再帰的にアラートを表示させるための関数
    internal func showRecursiveAuthInput(alert: NSAlert, input: NSTextField, window: NSWindow){
        alert.beginSheetModal(for: window, completionHandler: { (result) in
            if result == NSApplication.ModalResponse.alertSecondButtonReturn {
                return
            }
            if input.stringValue.isEmpty {
                return self.showRecursiveAuthInput(alert: alert, input: input, window: window)
            }
            // 認証処理
            // urlにリクエスト
            // ref: http://www.office-matsunaga.biz/ios/description.php?id=54
            let url = URL(string: "https://us-central1-zip-lp-prd.cloudfunctions.net/auth")
            var request = URLRequest(url: url!)
            request.addValue("text/plain", forHTTPHeaderField: "Content-Type")
            request.httpMethod = "POST"
            let authkey = input.stringValue
            let params:[String: Any] = [
                "authkey": authkey
            ]
            do {
                request.httpBody = try JSONSerialization.data(withJSONObject: params, options: .prettyPrinted)
            } catch {
                print("failed")
            }
            let session = URLSession.shared
            session.dataTask(with: request) {(data, response, error) in
                // カンマ区切りで複数条件指定　ひとつでもnilだったらelseになるらしい
                // ref: https://qiita.com/eielh/items/1d956fa578cf182783b5
                if error == nil, let data = data, let response = response as? HTTPURLResponse {
                    if response.statusCode != 200 {
                        // main threadでUIを触らないといけない urlへのrequest処理は他スレッドでし実行されている
                        DispatchQueue.main.sync {
                            alert.messageText = "認証エラー"
                            alert.informativeText = "入力した認証コードを確認の上、再度お試しください。"
                            self.showRecursiveAuthInput(alert: alert, input: input, window: window)
                        }
                        return
                    }
                    let json = try! JSONDecoder().decode(AuthResponseData.self, from: data) as AuthResponseData
                    // tokenを保存する
                    let defaults = UserDefaults.standard
                    let dict: Dictionary<String, Any> = [
                        "token": json.token,
                        "authkey": authkey
                    ]
                    defaults.set(dict, forKey: "token")
                    defaults.synchronize()
                } else {
                    // -1009はオフラインのエラー
                    if (error! as NSError).code == -1009 {
                        DispatchQueue.main.sync {
                            alert.messageText = "通信エラー"
                            alert.informativeText = "ネットワーク状況を確認の上、再度お試しください。"
                            self.showRecursiveAuthInput(alert: alert, input: input, window: window)
                        }
                    }
                    print("error:\(error)")
                }
                }.resume()
        })
    }
    
    internal func checkSession() {
        if Util().loadToken() == nil {
            // authkeyでまだ認証していないためtokenの定期チェックは不要
            return
        }
        
        let SALT = "A7D86A02F50A02FE430955A925CECA94EF947D80E4BE1DEA2F6F55424E88C18B"
        let loadedToken = Util().loadToken()
        let token = "\(loadedToken!)_\(SALT)"
        let url = URL(string: "https://us-central1-zip-lp-prd.cloudfunctions.net/session")
        var request = URLRequest(url: url!)
        request.addValue("text/plain", forHTTPHeaderField: "Content-Type")
        request.httpMethod = "POST"
        let params:[String: Any] = [
            "token": token
        ]
        do {
            request.httpBody = try JSONSerialization.data(withJSONObject: params, options: .prettyPrinted)
        } catch {
            print("failed")
        }
        let session = URLSession.shared
        session.dataTask(with: request) {(data, response, error) in
            if error == nil, let data = data, let response = response as? HTTPURLResponse {
                if response.statusCode == 403 {
                    print("Session Error. status is 403.")
                    UserDefaults.standard.removeObject(forKey: "token") // tokenがローカルとDBで一致しない時にはローカルのtokenを破棄する
                    return
                }
                print("session OK. status is not 403.")
            } else {
                print("error:\(error)")
            }
            }.resume()
    }
    
    public func createRandom(length: Int) -> String {
        let letters = "abcdefghijklmnopqrstuvwxyz0123456789"
        return String((0...length-1).map{ _ in letters.randomElement()! })
    }
    
    public func fixProhibitedCharacters(dir: String, ZIP_APP_TMP_DIR_NAME: String) {
        let fd = FileManager.default
        var fileNames = [String]()
        // 参考：https://121ware.com/qasearch/1007/doc/answer/image/008/008744/008744a.png
        let invalidStrings = ["\\", "<", ">", ":", "*", "?", "\"", "", "¥", "|"]
        
        FileManager().subpaths(atPath: dir)?.forEach {
            fileNames.append($0)
        }
        
        do {
            try fd.createDirectory(
                at: URL(fileURLWithPath: "\(dir)/\(ZIP_APP_TMP_DIR_NAME)/archive"),
                withIntermediateDirectories: true,
                attributes: nil
            )
        } catch let error as NSError {
            print("create dir error:\(error)")
        }
        
        for fileName in fileNames {
            do {
                try fd.copyItem(
                    atPath: "\(dir)/\(fileName)",
                    toPath: "\(dir)/\(ZIP_APP_TMP_DIR_NAME)/archive/\(self.fixFilePath(filePath: fileName))"
                )
            } catch let error as NSError {
                print("on fixProhibitedCharacters error:\(error), error fileName: \(fileName)")
            }
        }
        
        // 余分にコピーしてしまうファイルを削除
        FileManager().subpaths(atPath: "\(dir)/\(ZIP_APP_TMP_DIR_NAME)/archive")?.forEach {
            let path = "\(dir)/\(ZIP_APP_TMP_DIR_NAME)/archive/\($0)"
            
            let pathIsInvalid = invalidStrings.filter { (str: String) -> Bool in
                return path.contains(str)
            }
            
            if pathIsInvalid.count > 0 {
                do {
                    try fd.removeItem(
                        atPath: path
                    )
                } catch {
                    print("move unnecessary file error")
                }
            }
        }
    }
    
    private func fixFilePath(filePath: String) -> String {
        let invalidStrings:[String] = ["\\", "<", ">", ":", "*", "?", "\"", "", "¥", "|"]
        var _filePath = filePath
        
        invalidStrings.forEach {
            while(_filePath.range(of: $0) != nil){
                let range = _filePath.range(of: $0)
                _filePath.replaceSubrange(range!, with: "_")
            }
        }
        
        return _filePath
    }
}
