//
//  ViewController.swift
//  zip_app
//
//  Created by Michiyuki Abe on 2018/12/28.
//  Copyright © 2018 zip_pro. All rights reserved.
//

import Cocoa

class ViewController: NSViewController, DragAndDropDelegate {
    
    // Views
    @IBOutlet weak var fileDragDropTopLayerView: FileDragDropTopLayerView!
    @IBOutlet weak var imageView: NSImageView!
    @IBOutlet weak var encryptCheckBox: NSButton!
    @IBOutlet weak var passwordTextField: NSTextField!
    @IBOutlet weak var selectDirCheckBox: NSButton!
    @IBOutlet weak var randomButton: NSButton!
    var authAlert: NSAlert = NSAlert()
    var authInput: NSTextField = NSTextField(frame: NSRect(x: 0, y: 0, width: 300, height: 20))
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // delegateの設定
        fileDragDropTopLayerView.delegate = self
        
        // 初期化
        passwordTextField.isEnabled = false
        
        // 前回の設定読込と反映
        let settings = (UserDefaults.standard.dictionary(forKey: "settings") != nil) ? Util().loadSettings() : SettingStatus()
        if settings.isEncrypt {
            encryptCheckBox.state = NSControl.StateValue.on
            passwordTextField.isEnabled = true
        }
        if !settings.password.isEmpty {
            passwordTextField.stringValue = settings.password
        }
        if settings.isSelectDir {
            selectDirCheckBox.state = NSControl.StateValue.on
        }
        // updateの有無チェック
        checkUpdate()
    }
    
    override func viewDidDisappear() {
        saveSettings(settings: getSettingStatus())
    }
    
    override var representedObject: Any? {
        didSet {
            // Update the view, if already loaded.
        }
    }
    
    func onDragAndDrop(sender: NSDraggingInfo) {
        imageView.image = NSImage(named: "drop")
        
        let pboard = sender.draggingPasteboard
        var targetFilePaths = [String]()
        var _savePath:String = ""
        
        if let urls = pboard.readObjects(forClasses: [NSURL.self], options: nil) as? [URL] {
            for url in urls {
                targetFilePaths.append(url.path)
                if (_savePath == "") {
                    _savePath = url.deletingLastPathComponent().path
                }
            }
        } else {
            print("dropped files do not exist.")
        }
    
        if (Util().userIsTrialPeriod()) {
            ZipAction().createZipEntry(
                targetFilePaths: targetFilePaths,
                savePath: _savePath,
                settingStatus: getSettingStatus()
            )
            return
        }
        
        if Util().loadToken() == nil {
            Util().showAuthAlert(window: self.view.window!)
            return
        }
        
        ZipAction().createZipEntry(
            targetFilePaths: targetFilePaths,
            savePath: _savePath,
            settingStatus: getSettingStatus()
        )
    }
    func onDrag() {
        imageView.image = NSImage(named: "drop_active")
    }
    
    func offDrag() {
        imageView.image = NSImage(named: "drop")
    }
    @IBAction func onRandomBtnClick(_ sender: Any) {
        passwordTextField.stringValue = Util().createRandom(length: 16)
    }
    
    @IBAction func onEncryptClick(_ sender: Any) {
        if let b = sender as? NSButton {
            print(passwordTextField.stringValue) // これがないと入力された値がクリアされることがある
            if b.state.rawValue == 1 {
                passwordTextField.isEnabled = true
            } else {
                passwordTextField.isEnabled = false
            }
        }
        
    }
    
    // 設定状況の取得
    func getSettingStatus() -> SettingStatus {
        return SettingStatus(
            isEncrypt: encryptCheckBox.state.rawValue == 1 ? true : false,
            password: passwordTextField.stringValue,
            isSelectDir: selectDirCheckBox.state.rawValue == 1 ? true : false
        )
    }
    
    func saveSettings(settings: SettingStatus) -> Bool {
        let defaults = UserDefaults.standard
        let dict = settings.dictionaryForUserDefaults()
        defaults.set(dict, forKey: "settings")
        return defaults.synchronize()
    }
    
    func checkUpdate(){
        // urlにリクエスト
        // ref: http://www.office-matsunaga.biz/ios/description.php?id=54
        let url = URL(string: "https://us-central1-zip-lp-prd.cloudfunctions.net/appinfo")
        var request = URLRequest(url: url!)
        request.addValue("text/plain", forHTTPHeaderField: "Content-Type")
        let session = URLSession.shared
        session.dataTask(with: request) {(data, response, error) in
            // カンマ区切りで複数条件指定　ひとつでもnilだったらelseになるらしい
            // ref: https://qiita.com/eielh/items/1d956fa578cf182783b5
            if error == nil, let data = data, let response = response as? HTTPURLResponse {
                if response.statusCode != 200 {
                    print("error:\(error!)")
                    return
                }
                let newAppInfo = try! JSONDecoder().decode(AppInfo.self, from: data) as AppInfo
                let nowBuild = Int(Bundle.main.infoDictionary!["CFBundleVersion"] as! String)
                if let _ = nowBuild, nowBuild! < newAppInfo.versionCode {
                    DispatchQueue.main.sync {
                        let alert = NSAlert()
                        alert.alertStyle = .informational
                        alert.addButton(withTitle: "確認する")
                        alert.addButton(withTitle: "閉じる")
                        alert.messageText = "更新があります"
                        alert.informativeText = newAppInfo.message
                        let result = alert.runModal()
                        if result == NSApplication.ModalResponse.alertFirstButtonReturn {
                            NSWorkspace.shared.open(URL(string: newAppInfo.url)!)
                        }
                    }
                }
            } else {
                print("error:\(error!)")
            }
        }.resume()
    }
}
