//
//  ZipAction.swift
//  zip_app
//
//  Created by Michiyuki Abe on 2019/02/17.
//  Copyright © 2019 zip_pro. All rights reserved.
//

import Foundation
import Cocoa

let ZIP_APP_DIR_NAME:String = "34a928f190f714d60756129ad55f14f51de0867b22344d7bf3d6f77805b31225"
let ZIP_APP_TMP_DIR_NAME:String = "17A83E501F804A8551D27619DA0CF23C04474FC6946073962B915FFE86421A56"

public class ZipAction {
    internal func createZipEntry(
        targetFilePaths:[String],
        savePath:String,
        settingStatus: SettingStatus
    ) {
        
        let status = settingStatus
        
        if status.isSelectDir {
            self.createZipToSelectedByUserPath(
                targetFilePaths: targetFilePaths,
                savePath: savePath,
                status: status
            )
            return
        }
        
        self.createZip(
            targetFilePaths: targetFilePaths,
            savePath: savePath,
            status: status
        )
    }
    
    internal func createZip(
        targetFilePaths:[String],
        savePath:String,
        status: SettingStatus
        ) {
        let fd = FileManager.default
        let tmpDirectory = NSHomeDirectory() + "/.zipforwin"
        
        print("tmpDirectory: " + tmpDirectory)
        // tmp保存先ディレクトリに既にarchive/が存在したら、そいつをremoveする
        if (fd.fileExists(atPath: "\(tmpDirectory)/\(ZIP_APP_DIR_NAME)")) {
            do {
                try fd.removeItem(
                    atPath: "\(tmpDirectory)/\(ZIP_APP_DIR_NAME)"
                )
            } catch {
                print("move archive.zip error")
            }
        }
        
        // テンポラリーなディレクトリを作成
        do {
            try fd.createDirectory(
                at: URL(fileURLWithPath: "\(tmpDirectory)/\(ZIP_APP_DIR_NAME)"),
                withIntermediateDirectories: true,
                attributes: nil
            )
        } catch let error as NSError {
            print("create dir error:\(error)")
        }
        
        // ドロップされたファイルとディレクトリをテンポラリーなディレクトリにコピー
        for filePath in targetFilePaths {
            do {
                try fd
                    .copyItem(
                        atPath: filePath,
                        toPath:
                        "\(tmpDirectory)/\(ZIP_APP_DIR_NAME)/\(URL(fileURLWithPath: filePath).lastPathComponent)")
            } catch let error as NSError {
                print("copy file or directories error:\(error), error file Path: \(filePath)")
            }
        }
        
        // テンポラリーなディレクトリ内のディレクトリとファイルに禁止文字が含まれていたら_に置換する
        Util().fixProhibitedCharacters(dir: "\(tmpDirectory)/\(ZIP_APP_DIR_NAME)", ZIP_APP_TMP_DIR_NAME: ZIP_APP_TMP_DIR_NAME)
        
        // 名前が重ならないようにzipファイルのネーミングをする
        var archiveFileNum = 0
        var archiveFileIsDuplicated = true
        var archiveFileName = ""
        
        while archiveFileIsDuplicated {
            archiveFileName = "archive\(archiveFileNum == 0 ? "" : String(archiveFileNum)).zip"
            if (fd.fileExists(atPath: "\(savePath)/\(archiveFileName)")) {
                archiveFileNum = archiveFileNum + 1
            } else {
                archiveFileIsDuplicated = false
            }
        }
    
        // テンポラリーなディレクトリに入っているファイルタイトルをshift-jisにし、zip圧縮する
        // 圧縮タスクが終わった時にテンポラリーなディレクトリを削除する
        let php = Process()
        php.executableURL = URL(fileURLWithPath:"/bin/sh")
        
        php.arguments = [
            "-c",
            "cd \(tmpDirectory)/\(ZIP_APP_DIR_NAME)/17A83E501F804A8551D27619DA0CF23C04474FC6946073962B915FFE86421A56; php -r 'function isDirEmpty($dir) { if (!is_readable($dir)) return NULL; return (count(glob($dir . \"/*\"))); } $za = new ZipArchive(); $za->open(\"\(archiveFileName)\", ZIPARCHIVE::CREATE); $iterator = new RecursiveDirectoryIterator(\"archive\"); $iterator = new RecursiveIteratorIterator($iterator); $list = array(); foreach ($iterator as $fileinfo) { if ($fileinfo->isFile()) { if (strpos($fileinfo->getPathname(),\".DS_Store\") === false) { $za->addFile($fileinfo->getPathname(), iconv(\"UTF-8-MAC\", \"CP932//TRANSLIT\", $fileinfo->getPathname())); } } else { if (substr($fileinfo->getPathname(), -3) == \"/..\") { $dirName = substr($fileinfo->getPathname() , 0, strlen($fileinfo->getPathname()) - 2); if (isDirEmpty($dirName) == 0) { $za->addEmptyDir(iconv(\"UTF-8-MAC\", \"CP932//TRANSLIT\", $dirName)); }} } } $za->close();'"
        ]
        
        do {
            try php.run()
        } catch let error as NSError {
            print("convert charset and create zip error:\(error)")
        }
        
        // zipパスワードが完成したさコールバックが取得できないため、タイマー処理をする
        var zipIsExist = false
        Timer.scheduledTimer(
            withTimeInterval: 0.2,
            repeats: true,
            block: { (timer) in
                if (zipIsExist) {
                    // タイマーを止める
                    return
                }
                
                print("in scheduledTimer")
                print("\(tmpDirectory)/\(ZIP_APP_DIR_NAME)/\(ZIP_APP_TMP_DIR_NAME)/\(archiveFileName)")
                
                if (fd.fileExists(atPath: "\(tmpDirectory)/\(ZIP_APP_DIR_NAME)/\(ZIP_APP_TMP_DIR_NAME)/\(archiveFileName)")) {
                    zipIsExist = true
                    
                    if !status.isEncrypt {
                        do {
                            try
                                fd.moveItem(
                                    atPath: "\(tmpDirectory)/\(ZIP_APP_DIR_NAME)/\(ZIP_APP_TMP_DIR_NAME)/\(archiveFileName)",
                                    toPath: "\(savePath)/\(archiveFileName)"
                                )
                            
                                Util().checkSession()
                        } catch {
                            print("copy file or directories error:\(error), error file Path: \(savePath)")
                        }
                        return
                    }
                    
                    self.setPassword(
                        savePath: savePath,
                        status: status,
                        tmpDirectory: tmpDirectory,
                        archiveFileName: archiveFileName,
                        ZIP_APP_TMP_DIR_NAME: ZIP_APP_TMP_DIR_NAME
                    )
                    timer.invalidate()
                }
        })
    }
    
    // ユーザーが明示的に選択したpathにzipを作成する
    internal func createZipToSelectedByUserPath(
        targetFilePaths: [String],
        savePath: String,
        status: SettingStatus
    ) {
        let openPanel = NSOpenPanel()
        
        openPanel.canChooseDirectories = true
        openPanel.canCreateDirectories = false
        openPanel.canChooseFiles = false
        openPanel.begin(completionHandler:) { (result) in
            if result == NSApplication.ModalResponse.OK {
                if let openPanelURL = openPanel.url {
                    self.createZip(
                        targetFilePaths: targetFilePaths,
                        savePath: openPanel.url!.path,
                        status: status
                    )
                } else {
                    self.createZip(
                        targetFilePaths: targetFilePaths,
                        savePath: savePath,
                        status: status
                    )
                }
            }
        }
    }

    internal func setPassword(
        savePath: String,
        status: SettingStatus,
        tmpDirectory: String,
        archiveFileName: String,
        ZIP_APP_TMP_DIR_NAME: String
    ) {
        let _password = status.password
        let expect = Process()
        let fd = FileManager.default
        expect.executableURL = URL(fileURLWithPath:"/usr/bin/expect")
        
        expect.arguments = [
            "-c",
            "set timeout 10; spawn zipcloak \(tmpDirectory)/\(ZIP_APP_DIR_NAME)/\(ZIP_APP_TMP_DIR_NAME)/\(archiveFileName); expect \"Enter password:\"; sleep 1; send \"\(_password)\n\"; expect \"Verify password:\"; sleep 1; send \"\(_password)\n\"; sleep 1; spawn mv \(tmpDirectory)/\(ZIP_APP_DIR_NAME)/\(ZIP_APP_TMP_DIR_NAME)/\(archiveFileName) \(savePath)/; sleep 3; exit 0;"
        ]
        
        do {
            try expect.run()
            Util().checkSession()
        } catch let error as NSError {
            print("grant password error:\(error)")
        }
        
        var archiveIsExist = true
        Timer.scheduledTimer(
            withTimeInterval: 2.0,
            repeats: true,
            block: { (timer) in
                if (!archiveIsExist) {
                    // タイマーを止める
                    return
                }
                
                if (!fd.fileExists(atPath: "\(tmpDirectory)/\(ZIP_APP_DIR_NAME)/\(ZIP_APP_TMP_DIR_NAME)/\(archiveFileName)")) {
                    archiveIsExist = false
                    
                    do {
                        try fd.removeItem(
                            atPath: "\(tmpDirectory)/\(ZIP_APP_DIR_NAME)"
                        )
                    } catch let error as NSError {
                        print(error)
                    }
                    
                    timer.invalidate()
                }
        })
    }
}
